﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using CsvHelper;
using System.IO;
using System.IO.Ports;
using System.Globalization;
using System.Windows.Forms;

namespace CO2_Chamber_Dashboard
{
	public partial class Form1 : Form
	{
		private static string csvPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.Desktop), $"diyncubator-{DateTime.Now.ToFileTime()}.csv");
		private Single numdata;
		private Single pdata;
		private Single gdata;
		private bool take_data = false;

		public delegate void d1(string indata);
		public Form1()
		{
			InitializeComponent();
			string[] ports = SerialPort.GetPortNames();
			comboBox1.Items.AddRange(ports);
			timer1.Start();
			//var csvPath = Path.Combine(Environment.CurrentDirectory, $"diyncubator-{DateTime.Now.ToFileTime()}.csv");
			numdata = 0;
			gdata = 0;
			pdata = 0;

			textBox5.Text = csvPath;
		}

		public void Form1_Load(object sender, EventArgs e)
		{

			
		}

        private void onButton_Click(object sender, EventArgs e)
        {
			serialPort1.Write("A");
			textBox3.Text = "Heater Run";
		}

        private void buttonOff_Click(object sender, EventArgs e)
        {
			serialPort1.Write("a");
			textBox3.Text = "Heater Standby";
		}
		private void button4_Click(object sender, EventArgs e)
		{
			serialPort1.Write("B");
			textBox7.Text = "CO2 Run";
		}
		private void button5_Click(object sender, EventArgs e)
		{
			serialPort1.Write("b");
			textBox7.Text = "CO2 Standby";
		}
		private void button6_Click(object sender, EventArgs e)
		{
			serialPort1.Write("C");
			textBox8.Text = "Humidity Run";
		}
		private void button7_Click(object sender, EventArgs e)
		{
			serialPort1.Write("c");
			textBox8.Text = "Humidity Standby";
		}

		private void serialPort1_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
		{
			string indata = serialPort1.ReadLine();
			d1 writeit = new d1(Write2Form);
			Invoke(writeit, indata);
		}

		public void Write2Form(string indata)
        {
			char firstchar;
			firstchar = indata[0];

			switch(firstchar)
			{
			case 'v':
				numdata = Convert.ToSingle(indata.Substring(1));
				textBox1.Text = String.Format("{0:0.00}", numdata);
					//progressBar1.Value = Convert.ToInt16(indata.Substring(1));
				break;
			case 'p':
				pdata = Convert.ToSingle(indata.Substring(1));
				textBox2.Text = String.Format("{0:0.00}", pdata);
				if (chart1.Series["Series1"].Points.Count > 150)
				{
					chart1.Series["Series1"].Points.RemoveAt(0);
				}
				if (pdata <= 100)
				{
					chart1.Series["Series1"].Points.AddY(pdata);
				}
				break;
			case 'g':
				gdata = Convert.ToSingle(indata.Substring(1));
				textBox4.Text = String.Format("{0:0.00}", gdata);
				break;

			}
		}

        private void textBox2_TextChanged(object sender, EventArgs e)
        {

        }

        private void timer1_Tick(object sender, EventArgs e)
        {
			if (take_data)
			{
				var records = new List<ReadOut>
				{
					new ReadOut { Time = DateTime.UtcNow, Temp = numdata, CO2 = pdata, Humid = gdata },
				};
				var config = new CsvHelper.Configuration.CsvConfiguration(CultureInfo.InvariantCulture)
				{
					// Don't write the header again.
					HasHeaderRecord = false,
				};

				using (var stream = File.Open(csvPath, FileMode.Append))
				using (var writer = new StreamWriter(stream))
				using (var csv = new CsvWriter(writer, config))
				{
					csv.WriteRecords(records);
				}
			}
		}

        private void button1_Click(object sender, EventArgs e)
        {	
			try
			{
				serialPort1.PortName = comboBox1.Text;
				serialPort1.Open();
			}
			catch(Exception err)
            {
				MessageBox.Show(err.Message,"Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
		}

        private void label5_Click(object sender, EventArgs e)
        {

        }

        private void button2_Click(object sender, EventArgs e)
        {
			if (!take_data)
			{
				var records = new List<ReadOut>
				{
					new ReadOut { Time = DateTime.UtcNow, Temp = numdata, CO2 = pdata, Humid = gdata },
				};
				using (var writer = new StreamWriter(csvPath))
				using (var csv = new CsvWriter(writer, CultureInfo.InvariantCulture))
				{
					csv.WriteRecords(records);
				}
			}
			take_data = true;

			textBox6.Text = "Recording";
        }

        private void button3_Click(object sender, EventArgs e)
        {
			take_data = false;
			textBox6.Text = "Not Recording";
        }

        private void button8_Click(object sender, EventArgs e)
        {
            if (folderBrowserDialog1.ShowDialog() == DialogResult.OK)
            {
				//csvPath = folderBrowserDialog1.SelectedPath;
				csvPath = Path.Combine(folderBrowserDialog1.SelectedPath, $"diyncubator-{DateTime.Now.ToFileTime()}.csv");
				textBox5.Text = csvPath;

			}
        }
    }
}
public class ReadOut
{
	public DateTime Time { get; set; }
	public Single Temp { get; set; }
	public Single CO2 { get; set; }
	public Single Humid { get; set; }
}
